package com.persagy.iottransfer.communication.mina.tcp.client;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.mina.codec.MinaCodecFactory;
import com.persagy.iottransfer.communication.util.*;
import com.persagy.iottransfer.config.IotProperties;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.mina.core.future.ConnectFuture;
import org.apache.mina.core.service.IoConnector;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFactory;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.ssl.SslFilter;
import org.apache.mina.transport.socket.nio.NioSocketConnector;

import java.net.InetSocketAddress;

@Slf4j
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class TCPClientManager implements IClientManager {
    public final PacketBuffer<Packet> receList = new PacketBuffer<Packet>();
    String projectId;
    IotProperties.Client client;
    String ip;
    int port;
    String encoding;
    String ssl_protocol;
    String ssl_keystoretype;
    String ssl_algorithm;
    byte[] ssl_file_km;
    String ssl_password_km;
    byte[] ssl_file_tm;
    String ssl_password_tm;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean separate;
    boolean separateBytes;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;
    Integer IdleTime = null;
    boolean sessionClosed = false;
    IClientHandler handlerManager;
    ILogger logger;
    IoSession session;
    IoConnector connector;
    String localAddress = "()";
    int remark_count = 0;

    public TCPClientManager(String ip, int port, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public TCPClientManager(String ip, int port, String encoding, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public TCPClientManager(String ip, int port, String encoding, int Max_size, byte[] prefixBytes,
                            byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public TCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public TCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size, byte prefix,
                            byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public TCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size, byte[] prefixBytes,
                            byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public TCPClientManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
        this.separateBytes = false;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size,
                            byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size,
                            byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                            Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
        this.separateBytes = false;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                            Integer IdleTime, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                            Integer IdleTime, int Max_size, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                            String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                            String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
        this.separateBytes = false;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                            String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                            String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public TCPClientManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                            String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                            String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size, byte[] prefixBytes,
                            byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    @Override
    public void setHandlerManager(IClientHandler handlerManager) {
        this.handlerManager = handlerManager;
    }

    public void setLogger(ILogger logger) {
        this.logger = logger;
    }

    @Override
    public String getProjectId() {
        return this.projectId;
    }

    @Override
    public IotProperties.Client getClient() {
        return this.client;
    }

    @Override
    public String Name() {
        return "TCP Client " + this.ip + ":" + this.port + localAddress;
    }

    @Override
    public synchronized boolean IsSessionClosed() {
        return this.sessionClosed;
    }

    @Override
    public synchronized void SetSessionClosed(boolean value) {
        this.sessionClosed = value;
    }

    @Override
    public synchronized void Start() {
        if (this.session == null) {
            try {
                IoConnector connectorInner = new NioSocketConnector();
                ProtocolCodecFactory factory;
                if (this.separate) {
                    if (this.separateBytes) {
                        factory = new MinaCodecFactory(this.ip, this.port, this.encoding, this.aes_password,
                                this.compress, this.prefixBytes, this.suffixBytes);
                    } else {
                        factory = new MinaCodecFactory(this.ip, this.port, this.encoding, this.aes_password,
                                this.compress, this.prefix, this.suffix);
                    }
                } else {
                    factory = new MinaCodecFactory(this.ip, this.port, this.encoding, this.aes_password, this.compress);
                }
                if (this.ssl_file_km != null || this.ssl_file_tm != null) {
                    SslFilter sslFilter = new SslFilter(
                            MySslContextFactory.getContext(this.ssl_protocol, this.ssl_keystoretype, this.ssl_algorithm,
                                    this.ssl_file_km, this.ssl_file_tm, this.ssl_password_km, this.ssl_password_tm));
                    sslFilter.setUseClientMode(true);
                    connectorInner.getFilterChain().addFirst("sslFilter", sslFilter);
                }
                connectorInner.getFilterChain().addLast("codec", new ProtocolCodecFilter(factory));
                connectorInner.setHandler(new TCPClientIoHandler(this));
                if (this.IdleTime != null) {
                    connectorInner.getSessionConfig().setBothIdleTime(this.IdleTime);
                }

                ConnectFuture connectFuture = connectorInner.connect(new InetSocketAddress(this.ip, this.port));
                connectFuture.awaitUninterruptibly();
                if (connectFuture.isDone()) {
                    if (connectFuture.isConnected()) {
                        this.session = connectFuture.getSession();
                        this.localAddress = "(" + this.session.getLocalAddress().toString() + ")";
                        this.connector = connectorInner;
                    } else {
                        connectorInner.dispose();
                        throw new Exception("连接服务端"+this.ip+":"+this.port+"失败");
                    }
                } else {
                    connectorInner.dispose();
                    throw new Exception();
                }

                log.warn(this.Name() + "\tStart");
            } catch (Exception e) {
                log.error(ExceptionUtil.Instance().GetMessage(e));
            }
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new Packet(), Integer.MAX_VALUE);
        }
        if (this.session != null) {
            // CloseFuture closeFuture = this.session.close(true);
            // closeFuture.awaitUninterruptibly();
            this.session.close(true);
            this.session = null;

            this.connector.dispose();
            this.connector = null;
        }
    }

    @Override
    public void AppendToSend(Packet MyPackage) {
        if (this.session == null) {
            this.Start();
        }
        if (this.session != null) {
            this.session.write(MyPackage);
        }
    }

    public void AppendToRece(Packet MyPackage) {
        this.receList.offer(MyPackage, this.Max_size);
    }

    @Override
    public Packet PopRece() {
        Packet MyPackage = this.receList.poll();
        return MyPackage;
    }

    @Override
    public Packet takeRece() throws InterruptedException {
        Packet MyPackage = this.receList.take();
        return MyPackage;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }

}
