package com.persagy.iottransfer.communication.mina.tcp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.HexToByte;
import lombok.extern.slf4j.Slf4j;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.core.session.IoSession;

import java.net.SocketAddress;
import java.util.Date;

@Slf4j
public class TCPServerIoHandler extends IoHandlerAdapter {
    TCPServerManager manager;

    public TCPServerIoHandler(TCPServerManager manager) {
        this.manager = manager;
    }

    @Override
    public void sessionCreated(IoSession session) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        log.info(this.manager.Name() + "\tsessionCreated\t" + remoteAddress);
    }

    @Override
    public void sessionOpened(IoSession session) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        this.manager.SetConnectTime(remoteAddress, new Date());
        if (!this.manager.socketMap.containsKey(remoteAddress)) {
            this.manager.socketMap.put(remoteAddress, session);
        }
        log.warn(this.manager.Name() + "\tsessionOpened\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionOpened(session.getRemoteAddress());
        }
    }

    @Override
    public void sessionClosed(IoSession session) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        this.manager.SetConnectTime(remoteAddress, null);
        this.manager.socketMap.remove(remoteAddress);
        log.warn(this.manager.Name() + "\tsessionClosed\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionClosed(session.getRemoteAddress());
        }
    }

    @Override
    public void exceptionCaught(IoSession session, Throwable cause) throws Exception {
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.exceptionCaught(session.getRemoteAddress(), cause);
        }
    }

    @Override
    public void sessionIdle(IoSession session, IdleStatus status) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        log.warn(this.manager.Name() + "\tsessionIdle\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionIdle(session.getRemoteAddress());
        }
        session.close(true);
    }

    @Override
    public void messageReceived(IoSession session, Object message) throws Exception {
        Packet packet = (Packet) message;
        SocketAddress socketAddress = session.getRemoteAddress();
        this.manager.AppendToRece(socketAddress.toString(), packet);
        try {
            getProject(packet.packetString, socketAddress);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public void getProject(String message, SocketAddress socketAddress) {
        if (this.manager.project2IoSessionMap.containsKey(message.substring(0, 10))) {
            return;
        }
        this.manager.project2IoSessionMap.put(message.substring(0, 10), socketAddress.toString());
    }

    @Override
    public void messageSent(IoSession session, Object message) throws Exception {
        Packet packet = (Packet) message;
        SocketAddress socketAddress = session.getRemoteAddress();
        if (this.manager.encoding != null) {
            log.info(this.manager.Name() + "\tsend\t" + socketAddress.toString() + "\t" + packet.packetString);
        } else {
            log.info(this.manager.Name() + "\tsend\t" + socketAddress.toString()
                    + "\t" + HexToByte.byteToHex(packet.packetBytes));
        }
    }
}
