package com.persagy.iottransfer.communication.mina.udp.client;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.mina.codec.MinaCodecFactory;
import com.persagy.iottransfer.communication.util.*;
import com.persagy.iottransfer.config.IotProperties;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.mina.core.future.ConnectFuture;
import org.apache.mina.core.service.IoConnector;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFactory;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.transport.socket.nio.NioDatagramConnector;

import java.net.InetSocketAddress;

@Slf4j
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class UDPClientManager implements IClientManager {
    public final PacketBuffer<Packet> receList = new PacketBuffer<Packet>();
    String projectId;
    IotProperties.Client client;
    String ip;
    int port;
    String targetIp;
    int targetPort;
    String encoding;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean sessionClosed = false;
    IClientHandler handlerManager;
    ILogger logger;
    IoSession session;
    IoConnector connector;
    int remark_count = 0;

    public UDPClientManager(String targetIp, int targetPort, String encoding, int Max_size) {
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.Max_size = Max_size;
    }

    public UDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.Max_size = Max_size;
    }

    public UDPClientManager(String targetIp, int targetPort, String encoding, boolean compress, int Max_size) {
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    public UDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding, boolean compress,
                            int Max_size) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    public UDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding, String aes_password,
                            boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    @Override
    public void setHandlerManager(IClientHandler handlerManager) {
        this.handlerManager = handlerManager;
    }

    public void setLogger(ILogger logger) {
        this.logger = logger;
    }

    @Override
    public String getProjectId() {
        return this.projectId;
    }

    @Override
    public IotProperties.Client getClient() {
        return this.client;
    }

    @Override
    public String Name() {
        if (this.ip == null) {
            return "UDP Client " + this.targetIp + ":" + this.targetPort;
        } else {
            return "UDP Client " + this.ip + ":" + this.port;
        }
    }

    @Override
    public synchronized boolean IsSessionClosed() {
        return this.sessionClosed;
    }

    @Override
    public synchronized void SetSessionClosed(boolean value) {
        this.sessionClosed = value;
    }

    @Override
    public synchronized void Start() {
        if (this.session == null) {
            try {
                IoConnector connectorInner = new NioDatagramConnector();
                ProtocolCodecFactory factory;
                if (this.ip == null) {
                    factory = new MinaCodecFactory(this.targetIp, this.targetPort, this.encoding, this.aes_password,
                            this.compress);
                } else {
                    factory = new MinaCodecFactory(this.ip, this.port, this.encoding, this.aes_password, this.compress);
                }
                connectorInner.getFilterChain().addLast("codec", new ProtocolCodecFilter(factory));
                connectorInner.setHandler(new UDPClientIoHandler(this));
                ConnectFuture connectFuture;
                if (this.ip == null) {
                    connectFuture = connectorInner.connect(new InetSocketAddress(this.targetIp, this.targetPort));
                } else {
                    connectFuture = connectorInner.connect(new InetSocketAddress(this.targetIp, this.targetPort),
                            new InetSocketAddress(this.ip, this.port));
                }

                connectFuture.awaitUninterruptibly();
                if (connectFuture.isDone()) {
                    if (connectFuture.isConnected()) {
                        this.session = connectFuture.getSession();
                        this.connector = connectorInner;
                    } else {
                        connectorInner.dispose();
                        throw new Exception();
                    }
                } else {
                    connectorInner.dispose();
                    throw new Exception();
                }

                log.warn(this.Name() + "\tStart");
            } catch (Exception e) {
                log.error(ExceptionUtil.Instance().GetMessage(e));
            }
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new Packet(), Integer.MAX_VALUE);
        }
        if (this.session != null) {
            // CloseFuture closeFuture = this.session.close(true);
            // closeFuture.awaitUninterruptibly();
            this.session.close(true);
            this.session = null;

            this.connector.dispose();
            this.connector = null;
        }
    }

    @Override
    public void AppendToSend(Packet MyPackage) {
        if (this.session == null) {
            this.Start();
        }
        if (this.session != null) {
            this.session.write(MyPackage);
        }
    }

    public void AppendToRece(Packet MyPackage) {
        this.receList.offer(MyPackage, this.Max_size);
    }

    @Override
    public Packet PopRece() {
        Packet entity = this.receList.poll();
        return entity;
    }

    @Override
    public Packet takeRece() throws InterruptedException {
        Packet MyPackage = this.receList.take();
        return MyPackage;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }

}
