package com.persagy.iottransfer.communication.netty.tcp.client;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.HexToByte;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class NettyTCPClientHandler extends ChannelInboundHandlerAdapter {
    NettyTCPClientManager manager;

    public NettyTCPClientHandler(NettyTCPClientManager manager) {
        this.manager = manager;
    }

    @Override
    public void channelRegistered(ChannelHandlerContext ctx) throws Exception {
        super.channelRegistered(ctx);

        this.manager.SetSessionClosed(false);
        log.warn(this.manager.Name() + "\tsessionOpened");
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionOpened();
        }
    }

    @Override
    public void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
        super.channelUnregistered(ctx);

        log.warn(this.manager.Name() + "\tsessionClosed");
        // �쳣�ر�
        this.manager.Stop();
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionClosed();
        }
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object message) throws Exception {
        Packet packet = (Packet) message;
        this.manager.AppendToRece(packet);
        if (this.manager.encoding != null) {
            log.info(this.manager.Name() + "\trece\t" + packet.packetString);
        } else {
            log.info(this.manager.Name() + "\trece\t" + HexToByte.byteToHex(packet.packetBytes));
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.exceptionCaught(cause);
        }
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        super.userEventTriggered(ctx, evt);

        if (evt instanceof IdleStateEvent) {
            log.warn(this.manager.Name() + "\tsessionIdle");
            if (this.manager.handlerManager != null) {
                this.manager.handlerManager.sessionIdle();
            }
        }
    }

}