package com.persagy.iottransfer.communication.netty.tcp.server;

import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.ssl.SslHandler;
import io.netty.util.concurrent.Future;
import io.netty.util.concurrent.GenericFutureListener;
import lombok.extern.slf4j.Slf4j;

import javax.net.ssl.SSLSession;
import javax.security.cert.X509Certificate;
import java.net.SocketAddress;
import java.text.SimpleDateFormat;

@Slf4j
@SuppressWarnings("rawtypes")
public class SSLHandshakeListener implements GenericFutureListener {
    NettyTCPServerManager manager;
    ChannelHandlerContext ctx;

    public SSLHandshakeListener(NettyTCPServerManager manager, ChannelHandlerContext ctx) {
        this.manager = manager;
        this.ctx = ctx;
    }

    @Override
    public void operationComplete(Future future) throws Exception {
        SocketAddress socketAddress = ctx.channel().remoteAddress();
        String remoteAddress = socketAddress.toString();
        if (future.isSuccess()) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");

            SSLSession ss = ctx.pipeline().get(SslHandler.class).engine().getSession();
            StringBuffer sb = new StringBuffer();
            sb.append("\n" + "cipherSuite:" + ss.getCipherSuite());
            X509Certificate cert = ss.getPeerCertificateChain()[0];
            sb.append("\n" + "version:" + cert.getVersion());
            sb.append("\n" + "serialNumber:" + cert.getSerialNumber().toString(16));
            sb.append("\n" + "from:" + sdf.format(cert.getNotBefore()));
            sb.append("\n" + "to:" + sdf.format(cert.getNotAfter()));
            sb.append("\n" + "subjectDN:" + cert.getSubjectDN().getName());
            sb.append("\n" + "issuerDN:" + cert.getIssuerDN().getName());
            sb.append("\n" + "sigAlg:" + cert.getSigAlgName());

            log.warn(this.manager.Name() + "\t" + "handshake success" + "\t" + remoteAddress + sb);
        } else {
            log.warn(this.manager.Name() + "\t" + "handshake failure" + "\t" + remoteAddress);
        }
    }
}
