package com.persagy.iottransfer.communication.netty.udp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.communication.util.HexToByte;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;

import java.net.SocketAddress;
import java.util.List;

@Slf4j
public class NettyUDPServerHandler extends SimpleChannelInboundHandler<DatagramPacket> {
    NettyUDPServerManager manager;

    public NettyUDPServerHandler(NettyUDPServerManager manager) {
        this.manager = manager;
    }

    @Override
    public void channelRead0(ChannelHandlerContext ctx, DatagramPacket msg) throws Exception {
        int count = msg.content().readableBytes();
        byte[] bytes = new byte[count];
        msg.content().readBytes(bytes);
        String remoteAddress = msg.sender().toString();
        List<Packet> packetList = this.manager.decoder.decode(remoteAddress, bytes);
        for (int i = 0; i < packetList.size(); i++) {
            Packet packet = packetList.get(i);
            PacketEntity pe = new PacketEntity();
            pe.address = msg.sender().toString();
            pe.content = packet;
            this.manager.receList.offer(pe, this.manager.Max_size);

            if (this.manager.encoding != null) {
                log.info(this.manager.Name() + "\trece\t" + remoteAddress + "\t" + packet.packetString);
            } else {
                log.info(this.manager.Name() + "\trece\t" + remoteAddress + "\t"
                        + HexToByte.byteToHex(packet.packetBytes));
            }
        }
    }

    @Override
    public void channelRegistered(ChannelHandlerContext ctx) throws Exception {
        super.channelRegistered(ctx);

        log.warn(this.manager.Name() + "\tsessionOpened\t");
    }

    @Override
    public void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
        super.channelUnregistered(ctx);

        log.warn(this.manager.Name() + "\tsessionClosed\t");
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.exceptionCaught(ctx.channel().remoteAddress(), cause);
        }
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        super.userEventTriggered(ctx, evt);

        if (evt instanceof IdleStateEvent) {
            SocketAddress socketAddress = ctx.channel().remoteAddress();
            String remoteAddress = socketAddress.toString();
            log.warn(this.manager.Name() + "\tsessionIdle\t" + remoteAddress);
            ctx.channel().close();
        }
    }
}
