package com.persagy.iottransfer.communication.nio.tcp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.communication.util.ILogger;
import com.persagy.iottransfer.communication.util.IServerHandler;
import com.persagy.iottransfer.communication.util.IServerManager;
import com.persagy.iottransfer.communication.util.PacketBuffer;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class NIOTCPServerManager implements IServerManager {
    public final Map<String, Date> connectDateMap = new HashMap<String, Date>();
    public final Map<String, PacketBuffer<Packet>> sendListMap = new HashMap<String, PacketBuffer<Packet>>();
    public final PacketBuffer<PacketEntity> receList = new PacketBuffer<PacketEntity>();
    int buffer_size = 1024 * 16;
    long SelectTimeout = 1L;
    int SleepCount = 1000;
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean separate;
    boolean separateBytes;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;
    NIOTCPServerThread thread;

    ILogger logger;
    int remark_count = 0;

    public NIOTCPServerManager(String ip, int port, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NIOTCPServerManager(String ip, int port, String encoding, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NIOTCPServerManager(String ip, int port, String encoding, int Max_size, byte[] prefixBytes,
                               byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NIOTCPServerManager(String ip, int port, String encoding, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NIOTCPServerManager(String ip, int port, String encoding, boolean compress, int Max_size, byte prefix,
                               byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NIOTCPServerManager(String ip, int port, String encoding, boolean compress, int Max_size, byte[] prefixBytes,
                               byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NIOTCPServerManager(String ip, int port, String encoding, String aes_password, boolean compress,
                               int Max_size, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    @Override
    public void setHandlerManager(IServerHandler handlerManager) {
    }

    @Override
    public String Name() {
        return "TCP Server " + this.ip + ":" + this.port + " NIO";
    }

    @Override
    public synchronized void Start() {
        if (this.thread == null) {
            this.thread = new NIOTCPServerThread(this);
            this.thread.start();
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new PacketEntity(), Integer.MAX_VALUE);
        }
        if (this.thread != null) {
            this.thread.requestStop();
            this.thread = null;
        }
    }

    @Override
    public synchronized void AllowJustSend_UDP(boolean allow) {

    }

    @Override
    public synchronized String[] GetClientAddresss() {
        return this.connectDateMap.keySet().toArray(new String[0]);
    }

    @Override
    public synchronized Date GetConnectTime(String clientAddress) {
        if (this.connectDateMap.containsKey(clientAddress)) {
            return this.connectDateMap.get(clientAddress);
        }
        return null;
    }

    public synchronized void SetConnectTime(String clientAddress, Date date) {
        this.connectDateMap.remove(clientAddress);
        if (date != null) {
            this.connectDateMap.put(clientAddress, date);
        }
    }

    @Override
    public void AppendToSend(String clientAddress, Packet MyPackage) {
        if (!this.sendListMap.containsKey(clientAddress)) {
            this.sendListMap.put(clientAddress, new PacketBuffer<Packet>());
        }
        this.sendListMap.get(clientAddress).offer(MyPackage, this.Max_size);
    }

    public Packet PopSend(String clientAddress) {
        if (!this.sendListMap.containsKey(clientAddress)) {
            return null;
        }

        PacketBuffer<Packet> sendList = this.sendListMap.get(clientAddress);
        Packet MyPackage = sendList.poll();
        return MyPackage;
    }

    public void AppendToRece(String clientAddress, Packet MyPackage) {
        PacketEntity entity = new PacketEntity();
        entity.address = clientAddress;
        entity.content = MyPackage;
        this.receList.offer(entity, this.Max_size);
    }

    @Override
    public PacketEntity PopRece() {
        PacketEntity entity = this.receList.poll();
        return entity;
    }

    @Override
    public PacketEntity takeRece() throws InterruptedException {
        PacketEntity entity = this.receList.take();
        return entity;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void closeClient(String clientAddress) {
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }
}
