package com.persagy.iottransfer.communication.nio.udp.client;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.IClientHandler;
import com.persagy.iottransfer.communication.util.IClientManager;
import com.persagy.iottransfer.communication.util.PacketBuffer;
import com.persagy.iottransfer.config.IotProperties;

public class NIOUDPClientManager implements IClientManager {
    public final PacketBuffer<Packet> sendList = new PacketBuffer<Packet>();
    public final PacketBuffer<Packet> receList = new PacketBuffer<Packet>();
    String ip;
    IotProperties.Client client;
    String projectId;
    int port;
    String targetIp;
    int targetPort;
    String encoding;
    int buffer_size = 1024 * 16;
    int Max_size = 1000;
    long SelectTimeout = 1L;
    int SleepCount = 1000;
    boolean sessionClosed = false;
    NIOUDPClientThread thread;
    int remark_count = 0;

    public NIOUDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding, int buffer_size,
                               int Max_size, long SelectTimeout, int SleepCount) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.buffer_size = buffer_size;

        this.Max_size = Max_size;
        this.SelectTimeout = SelectTimeout;
        this.SleepCount = SleepCount;
    }

    @Override
    public void setHandlerManager(IClientHandler handlerManager) {
    }

    @Override
    public String Name() {
        return "UDP Client " + this.ip + ":" + this.port + " NIO";
    }

    @Override
    public synchronized boolean IsSessionClosed() {
        return this.sessionClosed;
    }

    @Override
    public synchronized void SetSessionClosed(boolean value) {
        this.sessionClosed = value;
    }

    @Override
    public synchronized void Start() {
        if (this.thread == null) {
            this.thread = new NIOUDPClientThread(this);
            this.thread.start();
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new Packet(), Integer.MAX_VALUE);
        }
        if (this.thread != null) {
            this.thread.requestStop();
            this.thread = null;
        }
    }

    @Override
    public void AppendToSend(Packet MyPackage) {
        this.sendList.offer(MyPackage, this.Max_size);
    }

    public Packet PopSend() {
        Packet MyPackage = this.sendList.poll();
        return MyPackage;
    }

    public void AppendToRece(Packet MyPackage) {
        this.receList.offer(MyPackage, this.Max_size);
    }

    @Override
    public Packet PopRece() {
        Packet MyPackage = this.receList.poll();
        return MyPackage;
    }

    @Override
    public Packet takeRece() throws InterruptedException {
        Packet MyPackage = this.receList.take();
        return MyPackage;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }

    @Override
    public String getProjectId() {
        return this.projectId;
    }

    @Override
    public IotProperties.Client getClient() {
        return this.client;
    }
}
