package com.persagy.iottransfer.communication.nio.udp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.communication.entity.UDPSendEntity;
import com.persagy.iottransfer.communication.util.IServerHandler;
import com.persagy.iottransfer.communication.util.IServerManager;
import com.persagy.iottransfer.communication.util.PacketBuffer;

import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class NIOUDPServerManager implements IServerManager {
    public final Map<String, Date> connectDateMap = new HashMap<String, Date>();
    public final Map<String, SocketAddress> socketMap = new HashMap<String, SocketAddress>();
    public final PacketBuffer<PacketEntity> sendList = new PacketBuffer<PacketEntity>();
    public final PacketBuffer<PacketEntity> receList = new PacketBuffer<PacketEntity>();
    String ip;
    int port;
    String encoding;
    int buffer_size = 1024 * 16;
    int Max_size = 1000;
    long SelectTimeout = 1L;
    int SleepCount = 1000;
    NIOUDPServerThread thread;

    boolean allowJustSend = false;
    int remark_count = 0;

    public NIOUDPServerManager(String ip, int port, String encoding, int buffer_size, int Max_size, long SelectTimeout,
                               int SleepCount) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.buffer_size = buffer_size;

        this.Max_size = Max_size;
        this.SelectTimeout = SelectTimeout;
        this.SleepCount = SleepCount;
    }

    @Override
    public void setHandlerManager(IServerHandler handlerManager) {
    }

    @Override
    public String Name() {
        return "UDP Server " + this.ip + ":" + this.port + " NIO";
    }

    @Override
    public synchronized void Start() {
        if (this.thread == null) {
            this.thread = new NIOUDPServerThread(this);
            this.thread.start();
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new PacketEntity(), Integer.MAX_VALUE);
        }
        if (this.thread != null) {
            this.thread.requestStop();
            this.thread = null;
        }
    }

    @Override
    public synchronized String[] GetClientAddresss() {
        return this.connectDateMap.keySet().toArray(new String[0]);
    }

    @Override
    public synchronized void AllowJustSend_UDP(boolean allow) {
        this.allowJustSend = allow;
    }

    @Override
    public synchronized Date GetConnectTime(String clientAddress) {
        if (this.connectDateMap.containsKey(clientAddress)) {
            return this.connectDateMap.get(clientAddress);
        }
        return null;
    }

    public synchronized void SetConnectTime(String clientAddress, Date date) {
        this.connectDateMap.remove(clientAddress);
        if (date != null) {
            this.connectDateMap.put(clientAddress, date);
        }
    }

    public synchronized void AddSocket(SocketAddress sa) {
        String serverAddress = sa.toString();
        if (!this.socketMap.containsKey(serverAddress)) {
            this.socketMap.put(serverAddress, sa);
        }
    }

    @Override
    public void AppendToSend(String clientAddress, Packet MyPackage) {
        PacketEntity entity = new PacketEntity();
        entity.address = clientAddress;
        entity.content = MyPackage;
        this.sendList.offer(entity, this.Max_size);
    }

    public UDPSendEntity PopSend() {
        PacketEntity entity = sendList.poll();
        if (entity != null) {
            if (this.allowJustSend && !this.socketMap.containsKey(entity.address)) {
                int index = entity.address.indexOf(':');
                String remote_ip = entity.address.substring(1, index);
                int remote_port = Integer.parseInt(entity.address.substring(index + 1));
                this.socketMap.put(entity.address, new InetSocketAddress(remote_ip, remote_port));
            }
            UDPSendEntity result = new UDPSendEntity();
            result.address = this.socketMap.get(entity.address);
            result.content = entity.content;
            return result;
        }
        return null;
    }

    public void AppendToRece(SocketAddress sa, Packet MyPackage) {
        String clientAddress = sa.toString();
        if (!this.socketMap.containsKey(clientAddress)) {
            this.socketMap.put(clientAddress, sa);
        }
        PacketEntity entity = new PacketEntity();
        entity.address = clientAddress;
        entity.content = MyPackage;
        this.receList.offer(entity, this.Max_size);
    }

    @Override
    public PacketEntity PopRece() {
        PacketEntity entity = this.receList.poll();
        return entity;
    }

    @Override
    public PacketEntity takeRece() throws InterruptedException {
        PacketEntity entity = this.receList.take();
        return entity;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void closeClient(String clientAddress) {
        this.socketMap.remove(clientAddress);
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }
}
