package com.persagy.iottransfer.communication.util;

import java.util.ArrayList;
import java.util.List;

public class MyStream {
    public int head_count = 0;
    boolean separateBytes = false;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;
    List<Byte> contents = new ArrayList<Byte>();
    boolean content_valid = false;
    List<Byte> buffer = new ArrayList<Byte>();

    public MyStream(byte prefix, byte suffix) {
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.head_count = 2;
    }

    public MyStream(byte[] prefixBytes, byte[] suffixBytes) {
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.head_count = this.prefixBytes.length + this.suffixBytes.length;
    }

    public List<byte[]> Process(byte[] bytes, int length) {
        List<byte[]> result = new ArrayList<byte[]>();
        if (this.separateBytes) {
            for (int i = 0; i < length; i++) {
                this.buffer.add(bytes[i]);
            }
            while (true) {
                int suffix_index = this.FindIndexForward(this.buffer, this.buffer.size(), this.suffixBytes);
                if (suffix_index == -1) {
                    break;
                }
                int prefix_index = this.FindIndexBackward(this.buffer, suffix_index, this.prefixBytes);
                if (prefix_index != -1) {
                    int bytes_length = suffix_index - prefix_index - this.prefixBytes.length;
                    if (bytes_length > 0) {
                        byte[] bytesInner = new byte[bytes_length];
                        for (int ii = 0; ii < bytes_length; ii++) {
                            bytesInner[ii] = this.buffer.get(prefix_index + this.prefixBytes.length + ii);
                        }
                        result.add(bytesInner);
                    }
                }
                for (int ii = 0; ii < suffix_index + this.suffixBytes.length; ii++) {
                    this.buffer.remove(0);
                }
            }
        } else {
            for (int i = 0; i < length; i++) {
                byte onebyte = bytes[i];
                if (onebyte == this.suffix) {
                    if (this.content_valid && this.contents.size() > 0) {
                        byte[] bytesInner = new byte[contents.size()];
                        for (int ii = 0; ii < this.contents.size(); ii++) {
                            bytesInner[ii] = this.contents.get(ii);
                        }
                        result.add(bytesInner);
                    }
                    this.contents.clear();
                    this.content_valid = false;
                } else if (onebyte == this.prefix) {
                    this.contents.clear();
                    this.content_valid = true;
                } else {
                    if (this.content_valid) {
                        this.contents.add(onebyte);
                    }
                }
            }
        }

        return result;
    }

    private int FindIndexForward(List<Byte> byteList, int length, byte[] bytes) {
        int result = -1;
        for (int i = 0; i <= length - bytes.length; i++) {
            if (this.Match(byteList, i, bytes)) {
                result = i;
                break;
            }
        }
        return result;
    }

    private int FindIndexBackward(List<Byte> byteList, int length, byte[] bytes) {
        int result = -1;
        for (int i = length - bytes.length; i >= 0; i--) {
            if (this.Match(byteList, i, bytes)) {
                result = i;
                break;
            }
        }
        return result;
    }

    private boolean Match(List<Byte> byteList, int index, byte[] bytes) {
        boolean match = true;
        for (int i = 0; i < bytes.length; i++) {
            if (byteList.get(index + i) != bytes[i]) {
                match = false;
                break;
            }
        }
        return match;
    }
}
