package com.persagy.iottransfer.kafka;

import com.alibaba.fastjson.JSONObject;
import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.server.IotServer;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.kafka.support.KafkaHeaders;
import org.springframework.messaging.handler.annotation.Header;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * @description:
 * @author:lgy
 * @data:2021/8/20 10:10
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "iot.type", name = "edge", havingValue = "true", matchIfMissing = false)
public class KafkaConsumerCloud2Edge {

    @KafkaListener(topics = KafkaProducer.TOPIC_COLLECT2EDGE, groupId = KafkaProducer.GROUP_IOT)
    public void topicCollect2Edge(ConsumerRecord<String, String> record, Acknowledgment ack, @Header(KafkaHeaders.RECEIVED_TOPIC) String topic) {
        Optional<String> message = Optional.ofNullable(record.value());
        if (message.isPresent()) {
            PacketEntity packetEntity = JSONObject.parseObject(message.get(), PacketEntity.class);
            String projectId = packetEntity.getProjectId();
            Packet packet = packetEntity.getContent();
            IotServer.tcpCollectServerManager.AppendToSendByProject(projectId, packet);
            log.info("topicCollect2Edge 消费了： Topic:" + topic + ",Message:" + packetEntity);
            ack.acknowledge();
        }
    }

    @KafkaListener(topics = KafkaProducer.TOPIC_CONTROL2EDGE, groupId = KafkaProducer.GROUP_IOT)
    public void topicControl2Edge(ConsumerRecord<String, String> record, Acknowledgment ack, @Header(KafkaHeaders.RECEIVED_TOPIC) String topic) {
        Optional<String> message = Optional.ofNullable(record.value());
        if (message.isPresent()) {
            PacketEntity packetEntity = JSONObject.parseObject(message.get(), PacketEntity.class);
            String projectId = packetEntity.getProjectId();
            Packet packet = packetEntity.getContent();
            IotServer.tcpControlServerManager.AppendToSendByProject(projectId, packet);
            log.info("topicControl2Edge 消费了： Topic:" + topic + ",Message:" + packetEntity);
            ack.acknowledge();
        }
    }
}