package com.persagy.iottransfer.kafka;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Component;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.ListenableFutureCallback;

/**
 * @description:
 * @author:lgy
 * @data:2021/8/20 10:11
 */
@Component
@Slf4j
public class KafkaProducer {

    //自定义topic
    public static final String TOPIC_EDGE2COLLECT = "persagy.iot.edge2collect";
    public static final String TOPIC_EDGE2CONTROL = "persagy.iot.edge2control";
    public static final String TOPIC_COLLECT2EDGE = "persagy.iot.collect2edge";
    public static final String TOPIC_CONTROL2EDGE = "persagy.iot.control2edge";
    //group
    public static final String GROUP_IOT = "persagy.group.iot";

    @Autowired
    private KafkaTemplate<String, Object> kafkaTemplate;

    public void sendMessage(PacketEntity obj, String topic) {
        String obj2String = JSONObject.toJSONString(obj);
        log.info("准备发送消息为：{}", obj2String);
        if (StrUtil.isEmpty(obj.getProjectId())) {
            log.info("项目id为空，丢去：{}", obj2String);
            return;
        }
        //发送消息
        ListenableFuture<SendResult<String, Object>> future = kafkaTemplate.send(topic, obj2String);
        future.addCallback(new ListenableFutureCallback<SendResult<String, Object>>() {
            @Override
            public void onFailure(Throwable throwable) {
                //发送失败的处理
                log.info(topic + " - 生产者 发送消息失败：" + throwable.getMessage());
            }

            @Override
            public void onSuccess(SendResult<String, Object> stringObjectSendResult) {
                //成功的处理
                log.info(topic + " - 生产者 发送消息成功：" + stringObjectSendResult.toString());
            }
        });
    }

    public void sendFromedge2collect(PacketEntity obj) {
        sendMessage(obj, TOPIC_EDGE2COLLECT);
    }

    public void sendFromedge2control(PacketEntity obj) {
        sendMessage(obj, TOPIC_EDGE2CONTROL);
    }

    public void sendFromcollect2edge(PacketEntity obj) {
        sendMessage(obj, TOPIC_COLLECT2EDGE);
    }

    public void sendFromcontrol2edge(PacketEntity obj) {
        sendMessage(obj, TOPIC_CONTROL2EDGE);
    }
}